<?php

namespace CodingAllNight\Proxy;

use Closure;
use Illuminate\Contracts\Config\Repository;

class TrustedProxies
{
    /**
     * The Config repository instance.
     *
     * @var \Illuminate\Contracts\Config\Repository
     */
    protected $config;

    /**
     * Create a new middleware instance.
     *
     * @param \Illuminate\Contracts\Config\Repository $config
     */
    public function __construct(Repository $config) {
        $this->config = $config;
    }

    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request $request
     * @param  \Closure $next
     * @return mixed
     */
    public function handle($request, Closure $next) {
        // Set trusted header names
        foreach ($this->getTrustedHeaders() as $headerKey => $headerName) {
            $request->setTrustedHeaderName($headerKey, $headerName);
        }

        $request->setTrustedProxies($this->getTrustedProxies($request->getClientIps()));

        return $next($request);
    }

    /**
     * Get the array of IP addresses for trusted proxies.
     *
     * @param array $clientIpAddresses Array of client IP addresses.
     * @return array An array of IP addresses of the trusted proxies.
     */
    protected function getTrustedProxies(array $clientIpAddresses = []) {
        $trustedProxies = $this->config->get('trustedproxies.proxies');

        // Trust all proxies
        if ($trustedProxies === '*') {
            return $clientIpAddresses;
        }

        return (array) $trustedProxies;
    }

    /**
     * Get the trusted header names.
     *
     * @return array An array of the trusted headers.
     */
    protected function getTrustedHeaders() {
        $trustedHeaderNames = $this->config->get('trustedproxies.headers');

        /*
         * If trustedproxies.headers is blank or empty, the defaults from
         * Symphony will be used.
         *
         * @see \Symfony\Component\HttpFoundation\Request::$trustedHeaders
         */
        return is_array($trustedHeaderNames) ? $trustedHeaderNames : [];
    }
}
